// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import BigInt from 'big-integer';
import superagent from 'superagent';
import C25519Point from './c25519Point';
import Hash from './hash';

export default class TObliviousClient {
    /**
     * @param {string} url 
     * @param {string} user 
     */
    constructor(url, user) {
        this.url = url + '/api';
        this.user = user;
        this._id = null;
    }

    get id() {
        return BigInt.fromArray(Array.from(Hash.shaBuffer((new URL(this.url)).host)), 256, false);
    }

    /** @param {C25519Point} pass */
    async GetSecret(pass) {
        var res = await superagent.get(`${this.url}/obliauth/${encodeBase64Url(this.user)}/secret/${encodeBase64Url(pass.toArray())}`);
        return C25519Point.from(Buffer.from(res.text, 'base64'));
    }

    /** @param {BigInt.BigInteger} secret */
    async SetSecret(secret) {
        var encoded = encodeBase64Url(Buffer.from(secret.toArray(256).value).toString('base64'));
        await superagent.post(`${this.url}/obliauth/${encodeBase64Url(this.user)}/secret/${encoded}`);
    }
}

/** @param {string|Uint8Array} input */
function encodeBase64Url(input) {
    const text = typeof input === "string" ? input : Buffer.from(input).toString('base64');
    return text.replace(/\=/g, "").replace(/\+/g, "-").replace(/\//g, "_");
}
